@@ -43,7 +43,6 @@
 #include "gtk-exif-entry-ascii.h"
 #include "gtk-exif-entry-copyright.h"
 #include "gtk-exif-entry-date.h"
-#include "gtk-exif-entry-exposure.h"
 #include "gtk-exif-entry-flash.h"
 #include "gtk-exif-entry-generic.h"
 #include "gtk-exif-entry-number.h"
@@ -143,7 +142,7 @@ gtk_exif_browser_init (GTypeInstance *instance, gpointer g_class)
 	gtk_object_sink (GTK_OBJECT (browser->priv->tooltips));
 
 	/* Placeholder */
-	browser->priv->empty = gtk_label_new ("Nothing selected.");
+	browser->priv->empty = gtk_label_new (_("Nothing selected."));
 	gtk_widget_show (browser->priv->empty);
 	g_object_ref (G_OBJECT (browser->priv->empty));
 }
@@ -157,18 +156,19 @@ gtk_exif_browser_get_content_list (GtkExifBrowser *b, ExifEntry *entry)
 	GtkWidget *swin, *viewport;
 	GtkExifContentList *list = NULL;
 
+	g_return_val_if_fail (GTK_EXIF_IS_BROWSER (b), NULL);
+	g_return_val_if_fail (entry != NULL, NULL);
+
 	n = g_list_length (b->priv->notebook->children);
 	for (i = 0; i < n; i++) {
 		swin = gtk_notebook_get_nth_page (b->priv->notebook, i);
+		if (!GTK_IS_SCROLLED_WINDOW (swin)) continue;
 		viewport = GTK_BIN (swin)->child;
 		list = GTK_EXIF_CONTENT_LIST (GTK_BIN (viewport)->child);
 		if (list->content == entry->parent)
 			break;
 	}
-	if (i == n)
-		return NULL;
-
-	return (list);
+	return (i == n) ? NULL : list;
 }
 
 static void
@@ -177,6 +177,8 @@ gtk_exif_browser_set_widget (GtkExifBrowser *browser, GtkWidget *w)
 	if (browser->priv->current)
 		gtk_container_remove (GTK_CONTAINER (browser->priv->info),
 				      browser->priv->current);
+	if (!w) return;
+
 	gtk_box_pack_start (GTK_BOX (browser->priv->info), w, TRUE, FALSE, 0);
 	browser->priv->current = w;
 }
@@ -210,6 +212,23 @@ on_entry_added (GtkExifEntry *entry, ExifEntry *e, GtkExifBrowser *b)
 static void
 on_entry_removed (GtkExifEntry *entry, ExifEntry *e, GtkExifBrowser *b)
 {
+	GtkExifContentList *list;
+
+	list = gtk_exif_browser_get_content_list (b, e);
+	if (!list) return;
+
+	switch (e->tag) {
+	case EXIF_TAG_RESOLUTION_UNIT:
+	case EXIF_TAG_X_RESOLUTION:
+	case EXIF_TAG_Y_RESOLUTION:
+		/* Do nothing. */
+		break;
+	default:
+		gtk_exif_browser_set_widget (b, b->priv->empty);
+		break;
+	}
+
+	gtk_exif_content_list_remove_entry (list, e);
 }
 
 static void
@@ -224,6 +243,11 @@ gtk_exif_browser_show_entry (GtkExifBrowser *browser, ExifEntry *entry)
 {
 	GtkWidget *w;
 
+	if (!entry) {
+		gtk_exif_browser_set_widget (browser, browser->priv->empty);
+		return;
+	}
+
 	switch (entry->tag) {
 	case EXIF_TAG_EXIF_VERSION:
 	case EXIF_TAG_FLASH_PIX_VERSION:
@@ -239,8 +263,6 @@ gtk_exif_browser_show_entry (GtkExifBrowser *browser, ExifEntry *entry)
 		w = gtk_exif_entry_flash_new (entry);
 		break;
 	case EXIF_TAG_EXPOSURE_PROGRAM:
-		w = gtk_exif_entry_exposure_new (entry);
-		break;
 	case EXIF_TAG_SENSING_METHOD:
 	case EXIF_TAG_ORIENTATION:
 	case EXIF_TAG_METERING_MODE:
@@ -391,7 +413,7 @@ gtk_exif_browser_show_thumbnail (GtkExifBrowser *b)
 				GTK_SCROLLED_WINDOW (b->priv->thumb), image);
 		}
 		g_object_unref (G_OBJECT (loader));
-		tip = g_strdup_printf (_("Size: %i byte(s)."),
+		tip = g_strdup_printf (_("Size: %i byte(s)."), 
 				       b->priv->data->size);
 		gtk_tooltips_set_tip (b->priv->tooltips, b->priv->thumb,
 				      tip, NULL);
