@@ -27,9 +27,11 @@
 #include <gtk/gtkradiobutton.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcelllayout.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkframe.h>
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkcombobox.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtklabel.h>
@@ -69,7 +71,7 @@ typedef struct _ResolutionUnitObjects ResolutionUnitObjects;
 struct _ResolutionUnitObjects
 {
 	GtkToggleButton *check;
-	GtkOptionMenu *menu;
+	GtkComboBox *menu;
 };
 
 struct _GtkExifEntryResolutionPrivate
@@ -126,31 +128,33 @@ gtk_exif_entry_resolution_init (GTypeInstance *instance, gpointer g_class)
 
 GTK_EXIF_CLASS (entry_resolution, EntryResolution, "EntryResolution")
 
-static void
-on_inch_activate (GtkMenuItem *item, GtkExifEntryResolution *entry)
-{
-	ExifEntry *e;
-	ExifByteOrder o;
+enum {
+	UNIT_CENTIMETER = 3,
+	UNIT_INCH = 2
+};
 
-	e = exif_content_get_entry (entry->priv->content,
-				    entry->priv->tag_u);
-	g_return_if_fail (e != NULL);
-	o = exif_data_get_byte_order (e->parent->parent);
-	exif_set_short (e->data, o, 2);
-	gtk_exif_entry_changed (GTK_EXIF_ENTRY (entry), e);
-}
+enum {
+	UNIT_COLUMN,
+	NAME_COLUMN,
+	N_COLUMNS
+};
 
 static void
-on_centimeter_activate (GtkMenuItem *item, GtkExifEntryResolution *entry)
+on_unit_changed (GtkComboBox *cb, GtkExifEntryResolution *entry)
 {
-	ExifEntry *e;
+	GtkTreeModel *tm = gtk_combo_box_get_model (cb);
+	GtkTreeIter iter;
+	GValue v = {0,};
 	ExifByteOrder o;
+	ExifEntry *e;
 
-	e = exif_content_get_entry (entry->priv->content,
-				    entry->priv->tag_u);
-	g_return_if_fail (e != NULL);
+	gtk_combo_box_get_active_iter (cb, &iter);
+	gtk_tree_model_get_value (tm, &iter, UNIT_COLUMN, &v);
+	
+	e = exif_content_get_entry (entry->priv->content, entry->priv->tag_u);
+	g_return_if_fail (e);
 	o = exif_data_get_byte_order (e->parent->parent);
-	exif_set_short (e->data, o, 3);
+	exif_set_short (e->data, o, g_value_get_int (&v));
 	gtk_exif_entry_changed (GTK_EXIF_ENTRY (entry), e);
 }
 
@@ -219,20 +223,23 @@ gtk_exif_entry_resolution_load_unit (GtkExifEntryResolution *entry,
 				     ExifEntry *e)
 {
 	ExifByteOrder o;
+	GtkTreeModel *tm;
+	GValue v = {0,};
+	GtkTreeIter iter;
 
 	o = exif_data_get_byte_order (e->parent->parent);
 	switch (e->format) {
 	case EXIF_FORMAT_SHORT:
-		switch (exif_get_short (e->data, o)) {
-		case 2:
-			gtk_option_menu_set_history (entry->priv->u.menu, 1);
-			break;
-		case 3:
-			gtk_option_menu_set_history (entry->priv->u.menu, 0);
-			break;
-		default:
-			g_warning ("Invalid unit!");
-		}
+		tm = gtk_combo_box_get_model (entry->priv->u.menu);
+		gtk_tree_model_get_iter_first (tm, &iter);
+		do {
+			gtk_tree_model_get_value (tm, &iter, UNIT_COLUMN, &v);
+			if (g_value_get_int (&v) ==
+					exif_get_short (e->data, o))
+				break;
+			g_value_unset (&v);
+		} while (gtk_tree_model_iter_next (tm, &iter));
+		gtk_combo_box_set_active_iter (entry->priv->u.menu, &iter);
 		break;
 	default:
 		g_warning ("Invalid format!");
@@ -301,16 +308,16 @@ on_cw_toggled (GtkToggleButton *toggle, GtkExifEntryResolution *entry)
 	gtk_widget_set_sensitive (entry->priv->ox.sq, toggle->active);
 	if (toggle->active && !e) { 
 		e = exif_entry_new ();
-		exif_entry_initialize (e, entry->priv->tag_x);
 		exif_content_add_entry (entry->priv->content, e);
+		exif_entry_initialize (e, entry->priv->tag_x);
 		gtk_exif_entry_resolution_load (entry, e);
 		exif_entry_unref (e);
 		gtk_exif_entry_added (GTK_EXIF_ENTRY (entry), e);
 	} else if (!toggle->active && e) {
-		exif_entry_ref (e);
-		exif_content_remove_entry (entry->priv->content, e);
+		g_object_ref (entry);
 		gtk_exif_entry_removed (GTK_EXIF_ENTRY (entry), e);
-		exif_entry_unref (e);
+		exif_content_remove_entry (entry->priv->content, e);
+		g_object_unref (entry);
 	}
 }
 
@@ -325,16 +332,16 @@ on_ch_toggled (GtkToggleButton *toggle, GtkExifEntryResolution *entry)
 	gtk_widget_set_sensitive (entry->priv->oy.sq, toggle->active);
         if (toggle->active && !e) {
 		e = exif_entry_new ();
-		exif_entry_initialize (e, entry->priv->tag_y);
 		exif_content_add_entry (entry->priv->content, e);
+		exif_entry_initialize (e, entry->priv->tag_y);
 		gtk_exif_entry_resolution_load (entry, e);
 		exif_entry_unref (e);
 		gtk_exif_entry_added (GTK_EXIF_ENTRY (entry), e);
         } else if (!toggle->active && e) {
-		exif_entry_ref (e);
-		exif_content_remove_entry (entry->priv->content, e);
+		g_object_ref (entry);
 		gtk_exif_entry_removed (GTK_EXIF_ENTRY (entry), e);
-		exif_entry_unref (e);
+		exif_content_remove_entry (entry->priv->content, e);
+		g_object_unref (entry);
         }
 }
 
@@ -349,16 +356,16 @@ on_unit_toggled (GtkToggleButton *toggle, GtkExifEntryResolution *entry)
 				  toggle->active);
 	if (toggle->active && !e) {
 		e = exif_entry_new ();
-		exif_entry_initialize (e, entry->priv->tag_u);
 		exif_content_add_entry (entry->priv->content, e);
+		exif_entry_initialize (e, entry->priv->tag_u);
 		gtk_exif_entry_resolution_load_unit (entry, e);
 		exif_entry_unref (e);
 		gtk_exif_entry_added (GTK_EXIF_ENTRY (entry), e);
 	} else if (!toggle->active && e) {
-		exif_entry_ref (e);
-		exif_content_remove_entry (entry->priv->content, e);
+		g_object_ref (entry);
 		gtk_exif_entry_removed (GTK_EXIF_ENTRY (entry), e);
-		exif_entry_unref (e);
+		exif_content_remove_entry (entry->priv->content, e);
+		g_object_unref (entry);
 	}
 }
 
@@ -366,9 +373,12 @@ GtkWidget *
 gtk_exif_entry_resolution_new (ExifContent *content, gboolean focal_plane)
 {
 	GtkExifEntryResolution *entry;
-	GtkWidget *hbox, *sp, *sq, *label, *menu, *item, *o, *c;
+	GtkWidget *hbox, *sp, *sq, *label, *o, *c;
 	GtkObject *ap, *aq;
 	ExifEntry *e;
+	GtkTreeIter iter;
+	GtkTreeModel *tm;
+	GtkCellRenderer *cell;
 
 	g_return_val_if_fail (content != NULL, NULL);
 
@@ -476,25 +486,27 @@ gtk_exif_entry_resolution_new (ExifContent *content, gboolean focal_plane)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (c), (e != NULL));
 	g_signal_connect (GTK_OBJECT (c), "toggled",
 			  G_CALLBACK (on_unit_toggled), entry);
-	o = gtk_option_menu_new ();
+	tm = GTK_TREE_MODEL (gtk_list_store_new (N_COLUMNS, G_TYPE_INT,
+						 G_TYPE_STRING));
+	gtk_list_store_append (GTK_LIST_STORE (tm), &iter);
+	gtk_list_store_set (GTK_LIST_STORE (tm), &iter,
+			    UNIT_COLUMN, UNIT_CENTIMETER,
+			    NAME_COLUMN, _("Centimeter"), -1);
+	gtk_list_store_append (GTK_LIST_STORE (tm), &iter);
+	gtk_list_store_set (GTK_LIST_STORE (tm), &iter,
+			    UNIT_COLUMN, UNIT_INCH,
+			    NAME_COLUMN, _("Inch"), -1);
+	o = gtk_combo_box_new_with_model (tm);
 	gtk_widget_show (o);
 	gtk_box_pack_start (GTK_BOX (hbox), o, TRUE, TRUE, 0);
-	entry->priv->u.menu = GTK_OPTION_MENU (o);
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
-	item = gtk_menu_item_new_with_label (_("Centimeter"));
-	gtk_widget_show (item);
-	gtk_container_add (GTK_CONTAINER (menu), item);
-	g_signal_connect (GTK_OBJECT (item), "activate",
-			  G_CALLBACK (on_centimeter_activate), entry);
-	item = gtk_menu_item_new_with_label (_("Inch"));
-	gtk_widget_show (item);
-	gtk_container_add (GTK_CONTAINER (menu), item);
-	g_signal_connect (GTK_OBJECT (item), "activate",
-			  G_CALLBACK (on_inch_activate), entry);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (o), menu);
-	if (e)
-		gtk_exif_entry_resolution_load_unit (entry, e);
+	cell = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (o), cell, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (o), cell,
+				"text", NAME_COLUMN, NULL);
+	g_signal_connect (G_OBJECT (o), "changed",
+			  G_CALLBACK (on_unit_changed), entry);
+	entry->priv->u.menu = GTK_COMBO_BOX (o);
+	if (e) gtk_exif_entry_resolution_load_unit (entry, e);
 
 	return (GTK_WIDGET (entry));
 }
